/* gdkcairo - replacing a gdkwindow with a cairo surface
 *
 * Copyright © 2003, 2004  Carl D. Worth <carl@theworths.org>
 *                         Evan Martin   <martine@danga.com>
 *                         Øyvind Kolås  <oeyvindk@hig.no>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <gdk/gdkx.h>
#include "gdkcairo.h"

static void
gdkcairo_init (gdkcairo_t *self, GtkWidget *widget)
{
    self->widget = widget;
    self->cr     = cairo_create ();


    self->backend = GDKCAIRO_BACKEND_IMAGE;
    #ifdef CAIRO_HAS_XLIB_SURFACE
    self->backend = GDKCAIRO_BACKEND_XLIB;
    #endif
    #ifdef USE_GL
    {
        char *GTKCAIRO_GL=getenv ("GTKCAIRO_GL");
        if (GTKCAIRO_GL && atoi (GTKCAIRO_GL))
            self->backend = GDKCAIRO_BACKEND_GL;
    }
    #endif
    {
        char *GDKCAIRO_BACKEND = getenv ("GTKCAIRO_BACKEND");
        if (GDKCAIRO_BACKEND) {
            if (!strcmp (GDKCAIRO_BACKEND, "image")) {
                self->backend = GDKCAIRO_BACKEND_IMAGE;
            }
            #ifdef CAIRO_HAS_XLIB_SURFACE
            else if (!strcmp (GDKCAIRO_BACKEND, "xlib")) {
                self->backend = GDKCAIRO_BACKEND_XLIB;
            }
            #endif
            #ifdef USE_GL
            else if (!strcmp (GDKCAIRO_BACKEND, "gl")) {
                self->backend = GDKCAIRO_BACKEND_GL;
            }
            #endif
            else {
                self->backend = GDKCAIRO_BACKEND_IMAGE;
                #ifdef CAIRO_HAS_XLIB_SURFACE
                self->backend = GDKCAIRO_BACKEND_XLIB;
                #endif
                fprintf (stderr, "unknown GTKCAIRO_BACKEND '%s' falling back\n", GDKCAIRO_BACKEND);
            }
        }
    }

    switch (self->backend) {
        case GDKCAIRO_BACKEND_IMAGE:
            break;
        #ifdef CAIRO_HAS_XLIB_SURFACE
        case GDKCAIRO_BACKEND_XLIB:
            break;
        #endif
        #ifdef USE_GL
        case GDKCAIRO_BACKEND_GL:
            self->glitz_surface = NULL;
            break;
        #endif
        default:
            g_assert (0);
            break;
    }
}

gdkcairo_t *
gdkcairo_new (GtkWidget *widget)
{
    gdkcairo_t *self = malloc (sizeof (gdkcairo_t));
    gdkcairo_init (self, widget);
    return self;
}

void
gdkcairo_destroy (gdkcairo_t *self)
{
	if (self->cr) {
		cairo_destroy (self->cr);
        self->cr = NULL;
	}
}

void
gdkcairo_realize (gdkcairo_t *self)
{
	GtkWidget *widget=self->widget;
	GdkWindowAttr attributes;
	gint attributes_mask;

	g_return_if_fail (widget != NULL);

	GTK_WIDGET_SET_FLAGS (self->widget, GTK_REALIZED);

	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = widget->allocation.width;
	attributes.height = widget->allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
	attributes.visual = gtk_widget_get_visual (widget);

retry:
    switch (self->backend) {
        case GDKCAIRO_BACKEND_IMAGE:
            break;
        #ifdef CAIRO_HAS_XLIB_SURFACE
        case GDKCAIRO_BACKEND_XLIB:
	      attributes.colormap = gtk_widget_get_colormap (widget);

	      attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
	      widget->window = gdk_window_new (widget->parent->window,
			    &attributes, attributes_mask);

            break;
        #endif
        #ifdef USE_GL
        case GDKCAIRO_BACKEND_GL: {
	        Display *dpy = gdk_x11_get_default_xdisplay ();
	        int screen = gdk_x11_get_default_screen ();
            XVisualInfo *vinfo;
	        glitz_format_t *onscreen_format;

	        onscreen_format = glitz_glx_find_standard_format (dpy, screen,
			    GLITZ_FORMAT_OPTION_ONSCREEN_MASK, GLITZ_STANDARD_ARGB32);
	        if (!onscreen_format)
	             onscreen_format = glitz_glx_find_standard_format (dpy, screen,
	             GLITZ_FORMAT_OPTION_ONSCREEN_MASK, GLITZ_STANDARD_RGB24);
            
	        /*  XSetWindowAttributes xswa;*/

	        if (onscreen_format) {
              vinfo = glitz_glx_get_visual_info_from_format (dpy, screen, onscreen_format);
              gtk_widget_set_double_buffered (widget, FALSE);
              attributes.visual = gdkx_visual_get (vinfo->visualid);
              attributes.colormap = gdk_colormap_new (attributes.visual, TRUE);

		      attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

		      widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
			                                   &attributes, attributes_mask);
                self->glitz_surface =
                glitz_glx_surface_create_for_window (dpy, screen,
					  onscreen_format,
					  gdk_x11_drawable_get_xid (widget->window));

              cairo_set_target_gl (self->cr, self->glitz_surface);
            } else {
            g_error ("could not find a usable GL visual\n");
            self->backend = GDKCAIRO_BACKEND_XLIB;
            goto retry;
             }
            }
            break;
        #endif
        default:
            break;
    }

    gtk_style_attach (widget->style, widget->window);
	gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
	gdk_window_set_user_data (widget->window, widget);
}

void
gdkcairo_size_allocate (gdkcairo_t *self,
                        int x,
                        int y,
                        int width,
                        int height)
{
	if (GTK_WIDGET_REALIZED (self->widget)) {
       gdk_window_move_resize (self->widget->window, x, y, width, height);

       switch (self->backend) {
         #ifdef CAIRO_HAS_XLIB_SURFACE
           case GDKCAIRO_BACKEND_XLIB:
               break;
         #endif
         #ifdef USE_GL
           case GDKCAIRO_BACKEND_GL:
            if (self->glitz_surface) {
				glitz_surface_update_size (self->glitz_surface);
            }
            break;
         #endif
           default:
              g_assert (0);
              break;
       }
    }
}

gint
gdkcairo_expose (gdkcairo_t *self,
                 GdkEventExpose *event)
{
    GtkWidget *widget = self->widget;

	g_return_val_if_fail (widget != NULL, FALSE);
	g_return_val_if_fail (event != NULL, FALSE);

    switch (self->backend) {
        #ifdef USE_GL
        case GDKCAIRO_BACKEND_GL:{

            cairo_rectangle (self->cr, 0, 0, widget->allocation.width, widget->allocation.height);
            cairo_set_rgb_color (self->cr, 0.7, 0.7, 0.7);  /* FIXME: use color from style */
            gtk_cairo_set_gdk_color (self->cr,
              &(self->widget->style->bg[GTK_STATE_NORMAL]) );
            cairo_fill      (self->cr);

	        g_signal_emit_by_name (self->widget, "paint", self->cr);
            glitz_surface_swap_buffers (self->glitz_surface);
            XSync (gdk_x11_get_default_xdisplay (), 0);
        }break;
        #endif
        #ifdef CAIRO_HAS_XLIB_SURFACE
        case GDKCAIRO_BACKEND_XLIB: {
            GdkDrawable *gdkdrawable;
            gint x_off, y_off;

	        gdk_window_get_internal_paint_info (widget->window,
			                                &gdkdrawable, &x_off, &y_off);

	        cairo_set_target_drawable (self->cr,
			                       gdk_x11_drawable_get_xdisplay (gdkdrawable),
			                       gdk_x11_drawable_get_xid (gdkdrawable));

	        /* counter offset of expose */
	        if (x_off || y_off) {
		        cairo_save (self->cr);
		        cairo_translate (self->cr, -x_off, -y_off);
	        }

	        g_signal_emit_by_name (self->widget, "paint", self->cr);

	        if (x_off || y_off) {
		        cairo_restore (self->cr);
            }
	    }break;
        #endif
        default:
            g_assert(0);
    }
	return FALSE;
}

/* vim: set ts=4 sw=4 et : */
